package de.uni_hildesheim.sse.trans.in.rsf;

import de.uni_hildesheim.sse.model.cst.ConstraintSyntaxTree;
import de.uni_hildesheim.sse.model.cst.OCLFeatureCall;
import de.uni_hildesheim.sse.model.varModel.DecisionVariableDeclaration;
import de.uni_hildesheim.sse.model.varModel.datatypes.OclKeyWords;
import de.uni_hildesheim.sse.trans.in.AbstractReader;
import de.uni_hildesheim.sse.trans.in.ParserException;

/**
 * Represents a condition read from a "Depends" line in a .rsf file.
 * The format looks like this: <br />
 * <code>
 * Depends <i>variable</i> <i>condition</i>
 * </code> <br />
 * <i>variable</i> can only be true, if <i>condition</i> is true. <br />
 * {@link RSFCondition#getConstraintSyntaxTreeHack(AbstractReader, String)} will convert it into: <br />
 * <code>
 * NOT <i>variable</i> OR <i>condition</i>
 * </code>
 * 
 * @author Adam Krafczyk
 */
class RSFDependsCondition extends RSFCondition {

    private String variable;
    private String condition;
    
    /**
     * Creates a condition with the given variable and condition as read from a "Depends" line in a .rsf file.
     * @param variable The name of the variable
     * @param condition The condition as read from the .rsf file
     */
    RSFDependsCondition(String variable, String condition) {
        this.variable = variable;
        this.condition = condition;
        // TODO error handling if only one is true?
        if (this.condition.startsWith("\"") && this.condition.endsWith("\"")) {
            this.condition = this.condition.substring(1, this.condition.length() - 1);
        }
    }
    
    @Override
    ConstraintSyntaxTree toConstraintSyntaxTree(AbstractReader reader) throws ParserException {
        DecisionVariableDeclaration varDecl = reader.getVariable(variable);
        OCLFeatureCall notVar = new OCLFeatureCall(varPool.obtainVariable(varDecl), OclKeyWords.NOT);
        
        ConstraintSyntaxTree condition = getConstraintSyntaxTree(reader, this.condition, variable);
        
        ConstraintSyntaxTree result = null;
        
        if (condition != null) {
            result = new OCLFeatureCall(notVar, OclKeyWords.OR, condition);
        }
        
        return result;
    }
    
}
